module Graph (
    Vertex, Weight,
    Graph,
    weight,
    readGraph,
    getGraph,
    fromAdjMatrix,
    toAdjMatrix,
    mkGraph
  ) where


{-# Language ScopedTypeVariables #-}

import qualified Data.ByteString.Lazy as BL
import qualified Data.Csv as CSV
import qualified Data.Vector as V
import Data.IntMap (IntMap)
import qualified Data.IntMap as Map
import Control.Monad(MonadPlus, guard)
import qualified Data.IntSet as IntSet
import Data.Array.Unboxed

type Vertex = Int
type Weight = Int

type Graph = IntMap (IntMap Weight)

weight :: Graph -> Vertex -> Vertex -> Maybe Weight
weight g i j = do
  jmap <- Map.lookup i g
  Map.lookup j jmap

insertEdge :: Vertex -> Vertex -> Weight -> Graph -> Graph
-- insertEdge i j w m = Map.insertWith Map.union i (Map.singleton j w) m
insertEdge i j w m = Map.insert i iarr m'
          where iarr = Map.insert j w ia
                ia = Map.findWithDefault Map.empty i m'
                m' = Map.insert j jarr m
                jarr = Map.insert i w ja
                ja = Map.findWithDefault Map.empty j m

readGraph :: IO (V.Vector (Int, Int))
readGraph = do
    let f = "/Users/saiteja/Documents/Columbia/Functional-Programming/playground/PolInc/git_web_ml/salaries.csv"
    BL.readFile f >>= dat . (CSV.decode CSV.NoHeader)
    where dat (Left err) = undefined
          dat (Right v) = return $ app v
            where app v = do
                            (v1 :: Int, v2 :: Int) <- v
                            return (v1, v2)

getEmptyGraph :: [Vertex] -> Graph
getEmptyGraph vertices = mat
        where 
        mat = foldr ins Map.empty edges
            where edges = [(i, j) | i <- vertices, j <- vertices]
                  ins (i,j) | i == j = insertEdge i i 0
                            | otherwise = insertEdge i j (maxBound :: Int)


getGraph :: [(Vertex, Vertex)] -> (Graph, [Vertex])
getGraph edges = (mat, vers)
    where 
        vers = getVertices edges
        mat = foldr ins Map.empty edges
        -- mat = foldr ins (getEmptyGraph vers) edges
            where ins (i,j) = insertEdge i j 1

getVertices' [] = []
getVertices' (x@(v1, v2):xs) = [v1, v2] ++ (getVertices' xs)

getVertices edges = IntSet.elems (IntSet.fromList $ getVertices' edges)

mkGraph :: [[Int]] -> Graph
mkGraph xss = Map.fromList (zipWith row [0..] xss)
  where
   row i xs = (i, Map.fromList [ (j, w) | (j,w) <- zip [0..] xs, w /= 100 ])

type AdjMatrix = UArray (Int,Int) Weight

toAdjMatrix :: Int -> Graph -> AdjMatrix
toAdjMatrix k m = accumArray (\_ e -> e) 999 ((0,0),(k,k))
      [ ((i,j),w) | (i,jmap) <- Map.toList m, (j,w) <- Map.toList jmap ]

fromAdjMatrix :: AdjMatrix -> [[Int]]
fromAdjMatrix m = chunk (k+1) (elems m)
  where
   (_, (k,_)) = bounds m

chunk :: Int -> [a] -> [[a]]
chunk _ [] = []
chunk n xs = as : chunk n bs
  where (as,bs) = splitAt n xs
